/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.bescheid;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;
import java.util.stream.Stream;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.historie.HistorieProcessor;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class BescheidHistorieProcessor implements HistorieProcessor {

	static final String BESCHEID_ITEM_NAME = "Bescheid";

	private final VorgangAttachedItemService vorgangAttachedItemService;

	private static final Predicate<Command> IS_NOT_CREATE_BESCHEID_COMMAND = Predicate
			.not(command -> command.getCommandOrder() == CommandOrder.CREATE_BESCHEID);
	private static final Predicate<Command> IS_NOT_DELETE_BESCHEID_COMMAND = Predicate
			.not(command -> command.getCommandOrder() == CommandOrder.DELETE_BESCHEID);
	private static final Predicate<Command> IS_UPDATE_BESCHEID_COMMAND = command -> command.getCommandOrder() == CommandOrder.UPDATE_BESCHEID;

	private static final Predicate<Command> IS_BESCHEID_ATTACHED_ITEM = command -> BESCHEID_ITEM_NAME
			.equals(MapUtils.getString(command.getBody(), VorgangAttachedItem.FIELD_ITEM_NAME, StringUtils.EMPTY));
	private static final Predicate<Command> IS_NOT_CREATE_ATTACHED_ITEM = Predicate
			.not(command -> command.getCommandOrder() == CommandOrder.CREATE_ATTACHED_ITEM
					&& IS_BESCHEID_ATTACHED_ITEM.test(command));
	private static final Predicate<Command> IS_NOT_UPDATE_ATTACHED_ITEM = Predicate
			.not(command -> command.getCommandOrder() == CommandOrder.UPDATE_ATTACHED_ITEM
					&& IS_BESCHEID_ATTACHED_ITEM.test(command));
	private static final Predicate<Command> IS_NOT_PATCH_ATTACHED_ITEM = Predicate
			.not(command -> command.getCommandOrder() == CommandOrder.PATCH_ATTACHED_ITEM
					&& IS_BESCHEID_ATTACHED_ITEM.test(command));

	@Override
	public List<Command> process(List<Command> commands) {
		var relevantCommands = filterByOrder(commands);
		return filterBescheidRelatedCommands(relevantCommands);
	}

	List<Command> filterByOrder(List<Command> commands) {
		return commands.stream()
				.filter(IS_NOT_CREATE_BESCHEID_COMMAND)
				.filter(IS_NOT_DELETE_BESCHEID_COMMAND)
				.filter(IS_NOT_CREATE_ATTACHED_ITEM)
				.filter(IS_NOT_UPDATE_ATTACHED_ITEM)
				.filter(IS_NOT_PATCH_ATTACHED_ITEM)
				.toList();
	}

	List<Command> filterBescheidRelatedCommands(List<Command> commands) {
		var excludeCommandIds = getExcludedBescheidRelatedCommandIds(commands);
		return commands.stream().filter(command -> !excludeCommandIds.contains(command.getId())).toList();
	}

	List<String> getExcludedBescheidRelatedCommandIds(List<Command> commands) {
		var sendBescheidCommands = commands.stream().filter(this::isSendBescheidCommand).toList();
		var sendBescheidCommandsRelationIds = sendBescheidCommands.stream().map(Command::getRelationId).toList();
		var orphanedUpdateBescheidCommands = commands.stream().filter(IS_UPDATE_BESCHEID_COMMAND)
				.filter(command -> !sendBescheidCommandsRelationIds.contains(command.getRelationId())).map(Command::getId);
		return Stream.concat(sendBescheidCommands.stream()
				.map(sendBescheidCommand -> getCommandIdsToFilter(sendBescheidCommand, commands))
				.flatMap(List::stream), orphanedUpdateBescheidCommands).toList();

	}

	private boolean isSendBescheidCommand(Command command) {
		return command.getCommandOrder() == CommandOrder.SEND_BESCHEID;
	}

	List<String> getCommandIdsToFilter(Command sendBescheidCommand, List<Command> commands) {
		var relatedUpdateBescheidCommands = getRelatedUpdateBescheidCommands(sendBescheidCommand, commands);
		if (isManualSent(getVorgangAttachedItem(sendBescheidCommand.getRelationId()).getItem())) {
			return Stream.concat(Stream.of(sendBescheidCommand.getId()), filterLatestByCreatedAt(relatedUpdateBescheidCommands)).toList();
		} else {
			return filterLatestByCreatedAt(relatedUpdateBescheidCommands).toList();
		}
	}

	List<Command> getRelatedUpdateBescheidCommands(Command command, List<Command> commands) {
		return commands.stream()
				.filter(filteredCommand -> StringUtils.equals(filteredCommand.getRelationId(), command.getRelationId()))
				.filter(IS_UPDATE_BESCHEID_COMMAND)
				.toList();
	}

	VorgangAttachedItem getVorgangAttachedItem(String id) {
		return vorgangAttachedItemService.getById(id);
	}

	boolean isManualSent(Map<String, ?> bescheid) {
		return getSendBy(bescheid) == SendBy.MANUAL;
	}

	private SendBy getSendBy(Map<String, ?> bescheid) {
		return SendBy.valueOf(MapUtils.getString(bescheid, Bescheid.SEND_BY_FIELD));
	}

	Stream<String> filterLatestByCreatedAt(List<Command> relatedUpdateBescheidCommands) {
		return relatedUpdateBescheidCommands.stream()
				.sorted(Comparator.comparing(Command::getCreatedAt, Comparator.naturalOrder()).reversed())
				.skip(1)
				.map(Command::getId);
	}
}