/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.downloadtoken;

import static de.ozgcloud.alfa.JwtTokenUtil.*;

import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Service;

import com.auth0.jwt.exceptions.JWTVerificationException;

import de.ozgcloud.alfa.JwtTokenUtil;
import de.ozgcloud.alfa.common.binaryfile.AlfaUserWithFileId;
import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserProfile;
import de.ozgcloud.common.errorhandling.TechnicalException;
import io.jsonwebtoken.Claims;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
class DownloadTokenService {

	@Autowired
	private CurrentUserService userService;

	@Autowired
	private JwtTokenUtil jwtTokenUtil;

	public String createToken(FileId fileId) {
		var user = userService.getUser();
		return jwtTokenUtil.generateToken(fileId, user);
	}

	public void handleToken(HttpServletRequest request, String token) {
		if (StringUtils.isNotBlank(token)) {
			try {
				jwtTokenUtil.verifyToken(token);

				SecurityContextHolder.getContext().setAuthentication(buildAuthentication(request, token));
			} catch (JWTVerificationException e) {
				LOG.error("JwtVerficationException in DownloadTokenService", e);
				throw new TechnicalException("download token not valid", e);
			}
		}
	}

	private Authentication buildAuthentication(HttpServletRequest request, String token) {
		var user = getUserFromToken(token);

		var authenticationToken = new UsernamePasswordAuthenticationToken(user, null, user.getAuthorities());
		authenticationToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));

		return authenticationToken;
	}

	AlfaUserWithFileId getUserFromToken(String token) {
		Optional<Claims> claimsOptional = jwtTokenUtil.getAllClaimsFromToken(token);
		var downloadUserBuilder = AlfaUserWithFileId.builder();
		claimsOptional.ifPresent(claims -> downloadUserBuilder.user(
						UserProfile.builder()
								.id(UserId.from(claims.get(USERID_CLAIM, String.class)))
								.firstName(claims.get(FIRSTNAME_CLAIM, String.class))
								.lastName(claims.get(LASTNAME_CLAIM, String.class))
								.authorities(jwtTokenUtil.getRolesFromToken(token))
								.organisationseinheitIds(jwtTokenUtil.getOrganisationseinheitIdsFromToken(token)).build())
				.fileId(FileId.from(claims.get(FILEID_CLAIM, String.class))));

		return downloadUserBuilder.build();
	}
}