/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import static de.ozgcloud.alfa.common.UserProfileUrlProviderTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.server.EntityLinks;

import de.ozgcloud.alfa.common.UserProfileUrlProvider;
import de.ozgcloud.alfa.common.UserProfileUrlProviderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class KommentarModelAssemblerTest {

	private final String PATH = KommentarController.KOMMENTAR_PATH + "/";
	private static final String REL_CREATED_BY = "createdBy";
	private static final String USER_MANAGER_URL = UserProfileUrlProviderTestFactory.ROOT_URL
			+ UserProfileUrlProviderTestFactory.USER_PROFILES_API_PATH;

	@Spy
	@InjectMocks
	private KommentarModelAssembler modelAssembler;

	@Mock
	private EntityLinks entityLinks;
	@Mock
	private VorgangController vorgangController;

	@Nested
	class TestToModel {

		@Test
		void shouldLoadVorgang() {
			when(vorgangController.getVorgang(VorgangHeaderTestFactory.ID)).thenReturn(VorgangWithEingangTestFactory.create());

			modelAssembler.toModel(KommentarTestFactory.create());

			verify(vorgangController).getVorgang(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldBuildModel() {
			var kommentar = KommentarTestFactory.create();
			var vorgang = VorgangWithEingangTestFactory.create();
			when(vorgangController.getVorgang(VorgangHeaderTestFactory.ID)).thenReturn(vorgang);

			modelAssembler.toModel(kommentar);

			verify(modelAssembler).buildModel(kommentar, vorgang);
		}
	}

	@Nested
	class TestLinksOnModel {
		private static final String COMMAND_BY_KOMMENTAR_PATH = //
				KommentarCommandController.KOMMENTAR_COMMANDS
						.replace("{kommentarId}", KommentarTestFactory.ID)
						.replace("{kommentarVersion}", String.valueOf(KommentarTestFactory.VERSION));

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@Test
		void shouldHaveSelfLink() {
			var link = buildModel().getLink(IanaLinkRelations.SELF);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo(PATH + KommentarTestFactory.ID);
		}

		@Test
		void shouldHaveEditLink() {
			when(vorgangController.isEditable(vorgang)).thenReturn(true);

			var model = buildModel();

			assertThat(model.getLink(KommentarModelAssembler.REL_EDIT)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(COMMAND_BY_KOMMENTAR_PATH);
		}

		@Test
		void shouldNotHaveEditLink() {
			when(vorgangController.isEditable(vorgang)).thenReturn(false);

			var model = buildModel();

			assertThat(model.getLink(KommentarModelAssembler.REL_EDIT)).isEmpty();
		}

		@Test
		void shouldHaveCreatedByLink() {
			UserProfileUrlProvider urlProvider = new UserProfileUrlProvider();
			initUserProfileUrlProvider(urlProvider);

			var model = buildModel();

			assertThat(model.getLink(REL_CREATED_BY)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(USER_MANAGER_URL + KommentarTestFactory.CREATED_BY);
		}

		@Test
		void shouldHaveAttachmentsLink() {
			var kommentar = KommentarTestFactory.create();
			var model = modelAssembler.buildModel(kommentar, VorgangWithEingangTestFactory.create());

			assertThat(model.getLink(KommentarModelAssembler.REL_ATTACHMENTS)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(String.format("%s%s/attachments", PATH, kommentar.getId()));
		}

		@Test
		void shouldNotHaveAttachmentsLinkIfNoAttachmentsAvailable() {
			var kommentar = KommentarTestFactory.createBuilder().clearAttachments().build();
			var model = modelAssembler.buildModel(kommentar, VorgangWithEingangTestFactory.create());

			assertThat(model.getLink(KommentarModelAssembler.REL_ATTACHMENTS)).isEmpty();
		}

		EntityModel<Kommentar> buildModel() {
			return modelAssembler.buildModel(KommentarTestFactory.create(), vorgang);
		}
	}

	@Nested
	class TestLinksOnCollectionModel {

		private final UserProfileUrlProvider urlProvider = new UserProfileUrlProvider();
		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@BeforeEach
		void beforeEach() {
			when(vorgangController.getVorgang(VorgangHeaderTestFactory.ID)).thenReturn(vorgang);
		}

		@Test
		void shouldBuildModel() {
			initUserProfileUrlProvider(urlProvider);
			var kommentar = KommentarTestFactory.create();
			var vorgang = VorgangWithEingangTestFactory.create();
			when(vorgangController.getVorgang(VorgangHeaderTestFactory.ID)).thenReturn(vorgang);

			modelAssembler.toCollectionModel(Stream.of(kommentar), VorgangHeaderTestFactory.ID);

			verify(modelAssembler).buildModel(kommentar, vorgang);
		}

		@Test
		void shouldLoadVorgang() {
			initUserProfileUrlProvider(urlProvider);

			modelAssembler.toCollectionModel(Collections.singleton(KommentarTestFactory.create()).stream(), VorgangHeaderTestFactory.ID);

			verify(vorgangController).getVorgang(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldHaveCreateKommentarLink() {
			initUserProfileUrlProvider(urlProvider);
			when(vorgangController.isEditable(vorgang)).thenReturn(true);

			var collectionModel = modelAssembler.toCollectionModel(Collections.singleton(KommentarTestFactory.create()).stream(),
					VorgangHeaderTestFactory.ID);

			var link = collectionModel.getLink(KommentarModelAssembler.REL_CREATE);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo(
					"/api/vorgangs/" + VorgangHeaderTestFactory.ID + "/kommentarCommands");
		}

		@Test
		void shouldNotHaveCreateKommentarLink() {
			initUserProfileUrlProvider(urlProvider);
			when(vorgangController.isEditable(vorgang)).thenReturn(false);

			var collectionModel = modelAssembler.toCollectionModel(Collections.singleton(KommentarTestFactory.create()).stream(),
					VorgangHeaderTestFactory.ID);

			var link = collectionModel.getLink(KommentarModelAssembler.REL_CREATE);
			assertThat(link).isEmpty();
		}

		@Test
		void shouldHaveUploadFileLink() {
			var model = modelAssembler.toCollectionModel(Stream.of(KommentarTestFactory.create()), VorgangHeaderTestFactory.ID);

			var link = model.getLink(KommentarModelAssembler.REL_UPLOAD_FILE);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo("/api/binaryFiles/" + VorgangHeaderTestFactory.ID + "/kommentarAttachment/file");
		}
	}
}