/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.context.ApplicationContext;
import org.springframework.core.env.Environment;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.server.EntityLinks;

import de.ozgcloud.alfa.common.UserProfileUrlProvider;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class WiedervorlageModelAssemblerTest {

	private final String PATH = WiedervorlageController.WIEDERVORLAGE_PATH + "/";

	@Spy
	@InjectMocks
	private WiedervorlageModelAssembler modelAssembler;

	@Mock
	private EntityLinks entityLinks;
	@Mock
	private VorgangController vorgangController;

	private UserProfileUrlProvider urlProvider = new UserProfileUrlProvider();

	@BeforeEach
	void initTest() {
		var context = mock(ApplicationContext.class);
		var environment = mock(Environment.class);
		when(environment.getProperty(anyString())).thenReturn("test/");
		when(context.getEnvironment()).thenReturn(environment);
		urlProvider.setApplicationContext(context);
	}

	@Nested
	class TestToModel {

		@Test
		void shouldLoadVorgang() {
			when(vorgangController.getVorgang(VorgangHeaderTestFactory.ID)).thenReturn(VorgangWithEingangTestFactory.create());

			modelAssembler.toModel(WiedervorlageTestFactory.create());

			verify(vorgangController).getVorgang(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldBuildModel() {
			var wiedervorlage = WiedervorlageTestFactory.create();
			var vorgang = VorgangWithEingangTestFactory.create();
			when(vorgangController.getVorgang(VorgangHeaderTestFactory.ID)).thenReturn(vorgang);

			modelAssembler.toModel(wiedervorlage);

			verify(modelAssembler).buildModel(wiedervorlage, vorgang);
		}
	}

	@Nested
	class TestLinksOnModel {
		private static final String COMMAND_BY_WIEDERVORLAGE_PATH = WiedervorlageCommandController.WIEDERVORLAGE_COMMANDS
				.replace("{wiedervorlageId}", WiedervorlageTestFactory.ID)
				.replace("{wiedervorlageVersion}", String.valueOf(WiedervorlageTestFactory.VERSION));

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@Test
		void shouldHaveSelfLink() {
			var link = buildModel().getLink(IanaLinkRelations.SELF);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo(PATH + WiedervorlageTestFactory.ID);
		}

		@Test
		void shouldHaveEditLink() {
			when(vorgangController.isEditable(vorgang)).thenReturn(true);

			var link = buildModel().getLink(WiedervorlageModelAssembler.REL_EDIT);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo(COMMAND_BY_WIEDERVORLAGE_PATH);
		}

		@Test
		void shouldNotHaveEditLink() {
			when(vorgangController.isEditable(vorgang)).thenReturn(false);

			var link = buildModel().getLink(WiedervorlageModelAssembler.REL_EDIT);

			assertThat(link).isEmpty();
		}

		@Nested
		class TestErledigenLink {

			@Test
			void shouldHaveLink() {
				var link = buildModel().getLink(WiedervorlageModelAssembler.REL_ERLEDIGEN);

				assertThat(link).isPresent();
				assertThat(link.get().getHref()).isEqualTo(COMMAND_BY_WIEDERVORLAGE_PATH);
			}

			@Test
			void shouldNotHaveLink() {
				EntityModel<Wiedervorlage> entityModel = modelAssembler.buildModel(WiedervorlageTestFactory.createBuilder().done(true).build(),
						VorgangWithEingangTestFactory.create());

				var link = entityModel.getLink(WiedervorlageModelAssembler.REL_ERLEDIGEN);

				assertThat(link).isNotPresent();
			}
		}

		@Nested
		class TestWiedereroeffnenLink {
			@Test
			void shouldHaveLink() {
				EntityModel<Wiedervorlage> entityModel = modelAssembler.buildModel(WiedervorlageTestFactory.createBuilder().done(true).build(),
						VorgangWithEingangTestFactory.create());

				var link = entityModel.getLink(WiedervorlageModelAssembler.REL_WIEDEREROEFFNEN);

				assertThat(link).isPresent();
				assertThat(link.get().getHref()).isEqualTo(COMMAND_BY_WIEDERVORLAGE_PATH);
			}

			@Test
			void shouldNotHaveLink() {
				EntityModel<Wiedervorlage> entityModel = modelAssembler.buildModel(WiedervorlageTestFactory.create(),
						VorgangWithEingangTestFactory.create());

				var link = entityModel.getLink(WiedervorlageModelAssembler.REL_WIEDEREROEFFNEN);

				assertThat(link).isNotPresent();
			}
		}

		@Nested
		class TestAttachmentsLink {

			@Test
			void shouldHaveLink() {
				EntityModel<Wiedervorlage> entityModel = modelAssembler
						.buildModel(WiedervorlageTestFactory.createBuilder().attachment(OzgFileTestFactory.ID).build(),
								VorgangWithEingangTestFactory.create());

				var link = entityModel.getLink(WiedervorlageModelAssembler.REL_ATTACHMENTS);

				assertThat(link).isPresent();
				assertThat(link.get().getHref()).isEqualTo("/api/wiedervorlages/" + WiedervorlageTestFactory.ID + "/attachments");
			}

			@Test
			void shouldNotHaveLink() {
				EntityModel<Wiedervorlage> entityModel = modelAssembler
						.buildModel(WiedervorlageTestFactory.createBuilder().clearAttachments().build(), VorgangWithEingangTestFactory.create());

				var link = entityModel.getLink(WiedervorlageModelAssembler.REL_ATTACHMENTS);

				assertThat(link).isNotPresent();
			}
		}

		private EntityModel<Wiedervorlage> buildModel() {
			return modelAssembler.buildModel(WiedervorlageTestFactory.create(), vorgang);
		}
	}

	@Nested
	class TestLinksOnCollectionModel {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@BeforeEach
		void beforeEach() {
			when(vorgangController.getVorgang(VorgangHeaderTestFactory.ID)).thenReturn(vorgang);
		}

		@Test
		void shouldBuildModel() {
			var wiedervorlage = WiedervorlageTestFactory.create();

			modelAssembler.toCollectionModel(Stream.of(wiedervorlage), VorgangHeaderTestFactory.ID);

			verify(modelAssembler).buildModel(wiedervorlage, vorgang);
		}

		@Test
		void shouldHaveCreateWiedervorlageLink() {
			when(vorgangController.isEditable(vorgang)).thenReturn(true);

			var collectionModel = modelAssembler.toCollectionModel(Collections.singleton(WiedervorlageTestFactory.create()).stream(),
					VorgangHeaderTestFactory.ID);
			var link = collectionModel.getLink(WiedervorlageModelAssembler.REL_CREATE);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo("/api/vorgangs/" + VorgangHeaderTestFactory.ID + "/wiedervorlageCommands");
		}

		@Test
		void shouldNotHaveCreateWiedervorlageLink() {
			when(vorgangController.isEditable(vorgang)).thenReturn(false);

			var collectionModel = modelAssembler.toCollectionModel(Collections.singleton(WiedervorlageTestFactory.create()).stream(),
					VorgangHeaderTestFactory.ID);
			var link = collectionModel.getLink(WiedervorlageModelAssembler.REL_CREATE);

			assertThat(link).isEmpty();
		}

		@Test
		void shouldHaveUploadFileLink() {
			var collectionModel = modelAssembler.toCollectionModel(Collections.singleton(WiedervorlageTestFactory.create()).stream(),
					VorgangHeaderTestFactory.ID);

			var link = collectionModel.getLink(WiedervorlageModelAssembler.REL_UPLOAD_FILE);

			assertThat(link).isPresent();
			assertThat(link.get().getHref()).isEqualTo("/api/binaryFiles/" + VorgangHeaderTestFactory.ID + "/wiedervorlageAttachment/file");
		}
	}
}